"""Service to send sample data to Wavefront.

It reads data from the AWS S3 bucket and sends it to Wavefront.

@author Yogesh Prasad Kurmi (ykurmi@vmware.com)
"""

import json
import logging
import os
import random
import boto3

from wavefront_sdk import WavefrontDirectClient


class SampleMetricsDI:
    """Read sample metrics from S3 and DI to WF cluster."""

    # pylint: disable=too-many-arguments
    def __init__(self,
                 iam_role_arn: str,
                 s3_bucket_name: str,
                 wf_cluster_url: str,
                 wf_cluster_token: str,
                 log_level='info') -> None:
        """Save inputs as class vars."""
        self.iam_role_arn = iam_role_arn
        self.s3_bucket_name = s3_bucket_name
        self.wf_cluster_url = wf_cluster_url
        self.wf_cluster_token = wf_cluster_token
        self.log_level = log_level

    def get_temp_creds(self) -> dict:
        """Create temporary AWS credentials using STS and IAM role."""
        # create an STS client for live connection to sts services
        sts_client = boto3.client('sts')

        # assume the role provided
        assume_role_obj = sts_client.assume_role(
            RoleArn=self.iam_role_arn,
            RoleSessionName="AssumeRoleSession1"
        )

        logging.info('********************************************')
        logging.info('assume_role_obj:\n%s', assume_role_obj)
        logging.info('********************************************')

        # return the temporary creds created in assume_role_obj
        return assume_role_obj['Credentials']

    def handler(self):
        """Read data from the AWS S3 bucket and sends it to Wavefront."""
        # Create Wavefront Direct Client
        wavefront_sender = WavefrontDirectClient(
            server=self.wf_cluster_url,
            token=self.wf_cluster_token
        )

        # get temporary AWS credentials
        temp_aws_creds = self.get_temp_creds()
        logging.info('********************************************')
        logging.info('temporary AWS creds:\n%s', temp_aws_creds)
        logging.info('********************************************')

        # Create S3 Client with temp AWS credentials
        s3_client = boto3.client(
            service_name='s3',
            aws_access_key_id=temp_aws_creds['AccessKeyId'],
            aws_secret_access_key=temp_aws_creds['SecretAccessKey'],
            aws_session_token=temp_aws_creds['SessionToken']
        )

        # Send service health metric
        wavefront_sender.send_metric(name="sample.data.service.health.status",
                                     value=1,
                                     timestamp=None,
                                     source="localhost",
                                     tags={"service": "running"}
                                     )

        objects = s3_client.list_objects(
            Bucket=self.s3_bucket_name
        )["Contents"]

        data_report_time = {}
        for data in objects:
            abs_path = data.get("Key")
            dir_path = "/".join(abs_path.split("/", 2)[:2])
            if ".json" in abs_path:
                iobj = s3_client.get_object(
                                Bucket=self.s3_bucket_name,
                                Key=abs_path
                       )
                idata = json.loads((iobj['Body'].read()).decode("utf-8"))
                for metric_info in idata:
                    # Try to send the same time stamp points
                    # to make the dashboard look like real
                    if dir_path not in data_report_time.keys():
                        point_count = len(metric_info.get("data"))
                        rand = random.randrange(point_count)
                        data_report_time[dir_path] = rand
                    try:
                        _data = metric_info.get("data")
                        mpoint = _data[data_report_time.get(dir_path)][1]
                    except IndexError:
                        mpoint = metric_info.get("data")[0][1]
                    wavefront_sender.send_metric(
                                        name=metric_info.get("label"),
                                        value=mpoint,
                                        timestamp=None,
                                        source=metric_info.get("host"),
                                        tags=metric_info.get("tags")
                    )
                logging.debug("Data sent for: %s", abs_path)

        wavefront_sender.close()
        logging.info("Done sending data to Wavefront.")


def enable_logging(log_level: str) -> None:
    """Enable logging object."""
    if log_level == "debug":
        logging.basicConfig(level=logging.DEBUG)
    else:
        # default
        logging.basicConfig(level=logging.INFO)


def main() -> None:
    """Entry point."""
    # get all environment variables we need
    s3_bucket_name = os.environ.get('S3_BUCKET_NAME')
    iam_role_arn = os.environ.get('IAM_ROLE_ARN')
    wf_cluster_url = os.environ.get('WF_CLUSTER_URL')
    wf_cluster_token = os.environ.get('WF_CLUSTER_TOKEN')
    try:
        log_level = os.environ["LOG_LEVEL"]
    except KeyError:
        log_level = 'info'

    # enable logging object
    enable_logging(log_level)

    # print our environment variables
    logging.info('s3_bucket_name: %s', s3_bucket_name)
    logging.info('iam_role_arn: %s', iam_role_arn)
    logging.info('wf_cluster_url: %s', wf_cluster_url)
    logging.info('wf_cluster_token: <HIDDEN>')
    logging.info('log_level: %s', log_level)

    # create our object
    smdi_obj = SampleMetricsDI(
                    iam_role_arn=iam_role_arn,
                    s3_bucket_name=s3_bucket_name,
                    wf_cluster_url=wf_cluster_url,
                    wf_cluster_token=wf_cluster_token,
                    log_level=log_level
    )

    # direct ingest sample metrics to wf cluster
    smdi_obj.handler()


if __name__ == "__main__":
    main()
